@@ -1,20 +1,21 @@
 # -*- coding: utf-8 -*-
 
+import xmlrpc.client
+import threading
+import zlib
+from os import sep
+import os
+import gettext
+
 import gi
 gi.require_version('Peas', '1.0')
 gi.require_version('Gtk', '3.0')
 gi.require_version('Totem', '1.0')
-from gi.repository import GLib, GObject # pylint: disable-msg=E0611
-from gi.repository import Peas, Gtk, Gdk # pylint: disable-msg=E0611
-from gi.repository import Gio, Pango, Totem # pylint: disable-msg=E0611
+from gi.repository import GLib, GObject # pylint: disable=wrong-import-position
+from gi.repository import Peas, Gtk, Gdk # pylint: disable=wrong-import-position,
+from gi.repository import Gio, Pango # pylint: disable=wrong-import-position,no-name-in-module
 
-import xmlrpc.client
-import threading
-import zlib
-from os import sep, path, mkdir
-import gettext
-
-from hash import hash_file
+from hash import hash_file # pylint: disable=wrong-import-position
 
 gettext.textdomain ("totem")
 
@@ -29,10 +30,12 @@ SUBTITLES_EXT = [
     "asc",
     "txt",
     "sub",
+    "vtt",
     "srt",
     "smi",
     "ssa",
     "ass",
+    "mpl",
 ]
 
 # Map of the language codes used by opensubtitles.org's API to their
@@ -146,14 +149,21 @@ LANGUAGES = {'sq':'alb',
          'uk':'ukr',
          'vi':'vie',}
 
+def _cache_subtitles_dir ():
+    bpath = GLib.get_user_cache_dir() + sep
+    bpath += 'totem' + sep + 'subtitles' + sep
+    return bpath
+
+# pylint: disable=R0902
 class SearchThread (threading.Thread):
     """
     This is the thread started when the dialog is searching for subtitles
     """
-    def __init__ (self, model, movie_hash, movie_size):
+    def __init__ (self, model, movie_hash, movie_size, movie_title):
         self._model = model
         self._movie_hash = movie_hash
         self._movie_size = movie_size
+        self._movie_title = movie_title
         self._done = False
         self._results = []
         self._lock = threading.Lock ()
@@ -161,39 +171,36 @@ class SearchThread (threading.Thread):
         threading.Thread.__init__ (self)
 
     def run (self):
-        self._lock.acquire (True)
-        (self._results,
-         self._message) = self._model.search_subtitles (self._movie_hash,
-                                                        self._movie_size)
-        self._done = True
-        self._lock.release ()
+        with self._lock:
+            (self._results,
+            self._message) = self._model.search_subtitles (self._movie_hash,
+                                                           self._movie_size,
+                                                           self._movie_title)
+            self._done = True
 
     def get_results (self):
         results = []
 
-        self._lock.acquire (True)
-        if self._done:
-            results = self._results
-        self._lock.release ()
+        with self._lock:
+            if self._done:
+                results = self._results
 
         return results
 
     def get_message (self):
-        message = _(u'Searching for subtitles…')
+        message = _('Searching for subtitles…')
 
-        self._lock.acquire (True)
-        if self._done:
-            message = self._message
-        self._lock.release ()
+        with self._lock:
+            if self._done:
+                message = self._message
 
         return message
 
     @property
     def done (self):
         """ Thread-safe property to know whether the query is done or not """
-        self._lock.acquire (True)
-        res = self._done
-        self._lock.release ()
+        with self._lock:
+            res = self._done
         return res
 
 class DownloadThread (threading.Thread):
@@ -210,43 +217,39 @@ class DownloadThread (threading.Thread):
         threading.Thread.__init__ (self)
 
     def run (self):
-        self._lock.acquire (True)
-        (self._subtitles,
-         self._message) = self._model.download_subtitles (self._subtitle_id)
-        self._done = True
-        self._lock.release ()
+        with self._lock:
+            (self._subtitles,
+             self._message) = self._model.download_subtitles (self._subtitle_id)
+            self._done = True
 
     def get_subtitles (self):
         subtitles = ''
 
-        self._lock.acquire (True)
-        if self._done:
-            subtitles = self._subtitles
-        self._lock.release ()
+        with self._lock:
+            if self._done:
+                subtitles = self._subtitles
 
         return subtitles
 
     def get_message (self):
-        message = _(u'Downloading the subtitles…')
+        message = _('Downloading the subtitles…')
 
-        self._lock.acquire (True)
-        if self._done:
-            message = self._message
-        self._lock.release ()
+        with self._lock:
+            if self._done:
+                message = self._message
 
         return message
 
     @property
     def done (self):
         """ Thread-safe property to know whether the query is done or not """
-        self._lock.acquire (True)
-        res = self._done
-        self._lock.release ()
+        with self._lock:
+            res = self._done
         return res
 
 # OpenSubtitles.org API abstraction
 
-class OpenSubtitlesModel (object):
+class OpenSubtitlesModel:
     """
     This contains the logic of the opensubtitles service.
     """
@@ -255,7 +258,7 @@ class OpenSubtitlesModel (object):
         self._token = None
 
         try:
-            import locale
+            import locale # pylint: disable=import-outside-toplevel
             (language_code, _) = locale.getlocale ()
             self.lang = LANGUAGES[language_code.split ('_')[0]]
         except (ImportError, IndexError, AttributeError, KeyError):
@@ -292,7 +295,7 @@ class OpenSubtitlesModel (object):
             if self._token:
                 return (True, '')
 
-        return (False, _(u'Could not contact the OpenSubtitles website'))
+        return (False, _('Could not contact the OpenSubtitles website'))
 
     def log_in (self, username='', password=''):
         """
@@ -304,14 +307,13 @@ class OpenSubtitlesModel (object):
         @rtype : (bool, string)
         """
 
-        self._lock.acquire (True)
-        result = self._log_in (username, password)
-        self._lock.release ()
+        with self._lock:
+            result = self._log_in (username, password)
 
         return result
 
-    def search_subtitles (self, movie_hash, movie_size):
-        self._lock.acquire (True)
+    def search_subtitles (self, movie_hash, movie_size, movie_title):
+        self._lock.acquire (True) # pylint: disable=R1732
 
         message = ''
 
@@ -325,13 +327,26 @@ class OpenSubtitlesModel (object):
                 result = self._server.SearchSubtitles (self._token,
                                                        [searchdata])
             except xmlrpc.client.ProtocolError:
-                message = _(u'Could not contact the OpenSubtitles website.')
+                message = _('Could not contact the OpenSubtitles website.')
 
             if result.get ('data'):
                 self._lock.release ()
                 return (result['data'], message)
-            else:
-                message = _(u'No results found.')
+
+            # Not found by movie_hash and moviebytesize, fallback to title
+            searchdata = {'sublanguageid': self.lang,
+                          'query'   : movie_title}
+            try:
+                result = self._server.SearchSubtitles (self._token,
+                                                       [searchdata])
+            except xmlrpc.client.ProtocolError:
+                message = _('Could not contact the OpenSubtitles website.')
+
+            if result.get ('data'):
+                self._lock.release ()
+                return (result['data'], message)
+
+            message = _('No results found.')
         else:
             message = log_in_message
 
@@ -340,10 +355,10 @@ class OpenSubtitlesModel (object):
         return (None, message)
 
     def download_subtitles (self, subtitle_id):
-        self._lock.acquire (True)
+        self._lock.acquire (True) # pylint: disable=R1732
 
         message = ''
-        error_message = _(u'Could not contact the OpenSubtitles website.')
+        error_message = _('Could not contact the OpenSubtitles website.')
 
         (log_in_success, log_in_message) = self._log_in ()
 
@@ -374,7 +389,7 @@ class OpenSubtitlesModel (object):
 
         return (None, message)
 
-class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
+class OpenSubtitles (GObject.Object, # pylint: disable=R0902
                      Peas.Activatable):
     __gtype_name__ = 'OpenSubtitles'
 
@@ -417,13 +432,17 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
 
         self._append_menu ()
 
-        self._totem.connect ('file-opened', self.__on_totem__file_opened)
-        self._totem.connect ('file-closed', self.__on_totem__file_closed)
+        self._totem.connect ('file-opened', self.__on_totem__file_opened) # pylint: disable=no-member
+        self._totem.connect ('file-closed', self.__on_totem__file_closed) # pylint: disable=no-member
 
         # Obtain the ServerProxy and init the model
         server = xmlrpc.client.Server ('http://api.opensubtitles.org/xml-rpc')
         self._model = OpenSubtitlesModel (server)
 
+        mrl = self._totem.get_current_mrl () # pylint: disable=E1101
+        if mrl:
+            self.__on_totem__file_opened(self._totem, self._totem.get_current_mrl ()) # pylint: disable=E1101
+
     def do_deactivate (self):
         if self._dialog:
             self._dialog.destroy ()
@@ -434,10 +453,8 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
     # UI related code
 
     def _build_dialog (self):
-        builder = Totem.plugin_load_interface ("opensubtitles",
-                                               "opensubtitles.ui", True,
-                                               self._totem.get_main_window (),
-                                               None)
+        ui_file_path = os.path.dirname (os.path.abspath (__file__)) + sep + 'opensubtitles.ui'
+        builder = Gtk.Builder.new_from_file (ui_file_path)
 
         # Obtain all the widgets we need to initialize
         combobox = builder.get_object ('language_combobox')
@@ -474,18 +491,18 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         renderer = Gtk.CellRendererText ()
         self._tree_view.set_model (self._list_store)
         renderer.set_property ('ellipsize', Pango.EllipsizeMode.END)
-        column = Gtk.TreeViewColumn (_(u"Subtitles"), renderer, text=0)
+        column = Gtk.TreeViewColumn (_("Subtitles"), renderer, text=0)
         column.set_resizable (True)
         column.set_expand (True)
         self._tree_view.append_column (column)
         # translators comment:
         # This is the file-type of the subtitle file detected
-        column = Gtk.TreeViewColumn (_(u"Format"), renderer, text=1)
+        column = Gtk.TreeViewColumn (_("Format"), renderer, text=1)
         column.set_resizable (False)
         self._tree_view.append_column (column)
         # translators comment:
         # This is a rating of the quality of the subtitle
-        column = Gtk.TreeViewColumn (_(u"Rating"), renderer, text=2)
+        column = Gtk.TreeViewColumn (_("Rating"), renderer, text=2)
         column.set_resizable (False)
         self._tree_view.append_column (column)
 
@@ -498,8 +515,7 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         # Set up signals
 
         combobox.connect ('changed', self.__on_combobox__changed)
-        self._dialog.connect ('delete-event', self._dialog.hide_on_delete)
-        self._dialog.set_transient_for (self._totem.get_main_window ())
+        self._dialog.set_transient_for (self._totem.get_main_window ()) # pylint: disable=no-member
         self._dialog.set_position (Gtk.WindowPosition.CENTER_ON_PARENT)
 
         # Connect the callbacks
@@ -510,7 +526,7 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         self._tree_view.connect ('row-activated',
                                self.__on_treeview__row_activate)
 
-    def _show_dialog (self, params, _):
+    def _show_dialog (self, *_):
         if not self._dialog:
             self._build_dialog ()
 
@@ -521,23 +537,22 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
     def _append_menu (self):
         self._action = Gio.SimpleAction.new ("opensubtitles", None)
         self._action.connect ('activate', self._show_dialog)
-        self._totem.add_action (self._action)
-        self._totem.set_accels_for_action ("app.opensubtitles",
+        self._totem.add_action (self._action) # pylint: disable=no-member
+        self._totem.set_accels_for_action ("app.opensubtitles",  # pylint: disable=no-member
                                            ["<Primary><Shift>s"])
 
-        menu = self._totem.get_menu_section ("subtitle-download-placeholder")
-        menu.append (_(u'_Download Movie Subtitles…'), "app.opensubtitles")
+        menu = self._totem.get_menu_section ("subtitle-download-placeholder") # pylint: disable=no-member
+        menu.append (_('_Download Movie Subtitles…'), "app.opensubtitles")
 
-        self._action.set_enabled (self._totem.is_playing () and
+        self._action.set_enabled (self._totem.is_playing () and # pylint: disable=no-member
                   self._check_allowed_scheme () and
                                   not self._check_is_audio ())
 
     def _check_allowed_scheme (self):
-        current_file = Gio.file_new_for_uri (self._totem.get_current_mrl ())
+        current_file = Gio.file_new_for_uri (self._totem.get_current_mrl ()) # pylint: disable=no-member
         scheme = current_file.get_uri_scheme ()
 
-        if (scheme == 'dvd' or scheme == 'http' or
-            scheme == 'dvb' or scheme == 'vcd'):
+        if scheme in ('dvd', 'http', 'dvb', 'vcd'):
             return False
 
         return True
@@ -546,15 +561,15 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         # FIXME need to use something else here
         # I think we must use video widget metadata but I don't found a way
         # to get this info from python
-        filename = self._totem.get_current_mrl ()
+        filename = self._totem.get_current_mrl () # pylint: disable=no-member
         if Gio.content_type_guess (filename, '')[0].split ('/')[0] == 'audio':
             return True
         return False
 
     def _delete_menu (self):
-        self._totem.empty_menu_section ("subtitle-download-placeholder")
+        self._totem.empty_menu_section ("subtitle-download-placeholder") # pylint: disable=no-member
 
-    def _get_results (self, movie_hash, movie_size):
+    def _get_results (self, movie_hash, movie_size, movie_title):
         self._list_store.clear ()
         self._apply_button.set_sensitive (False)
         self._find_button.set_sensitive (False)
@@ -562,11 +577,11 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         cursor = Gdk.Cursor.new (Gdk.CursorType.WATCH)
         self._dialog.get_window ().set_cursor (cursor)
 
-        thread = SearchThread (self._model, movie_hash, movie_size)
+        thread = SearchThread (self._model, movie_hash, movie_size, movie_title)
         thread.start ()
         GLib.idle_add (self._populate_treeview, thread)
 
-        self._progress.set_text (_(u'Searching subtitles…'))
+        self._progress.set_text (_('Searching subtitles…'))
         GLib.timeout_add (350, self._progress_bar_increment, thread)
 
     def _populate_treeview (self, search_thread):
@@ -597,33 +612,28 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
             subtitle_id = model.get_value (subtitle_iter, 3)
             subtitle_format = model.get_value (subtitle_iter, 1)
 
-            bpath = self._cache_subtitles_dir()
+            bpath = _cache_subtitles_dir()
 
             directory = Gio.file_new_for_path (bpath)
             try:
-                directory.make_directory_with_parents (None);
-            except:
+                directory.make_directory_with_parents (None)
+            except: # pylint: disable=bare-except
                 pass
 
             thread = DownloadThread (self._model, subtitle_id)
             thread.start ()
             GLib.idle_add (self._save_subtitles, thread, subtitle_format)
 
-            self._progress.set_text (_(u'Downloading the subtitles…'))
+            self._progress.set_text (_('Downloading the subtitles…'))
             GLib.timeout_add (350, self._progress_bar_increment, thread)
         else:
             #warn user!
             pass
 
-    def _cache_subtitles_dir (self):
-       bpath = GLib.get_user_cache_dir() + sep
-       bpath += 'totem' + sep + 'subtitles' + sep
-       return bpath
-
     def _movie_dir (self):
-       directory = Gio.file_new_for_uri (self._filename)
-       parent = directory.get_parent()
-       return parent.get_path ()
+        directory = Gio.file_new_for_uri (self._filename)
+        parent = directory.get_parent()
+        return parent.get_path ()
 
     def _save_subtitles (self, download_thread, extension):
         if not download_thread.done:
@@ -638,31 +648,33 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
             # Delete all previous cached subtitle for this file
             for ext in SUBTITLES_EXT:
                 # In the cache dir
-                old_subtitle_file = Gio.file_new_for_path (self._cache_subtitles_dir() + sep + movie_name + '.' + ext)
+                old_subtitle_file = Gio.file_new_for_path (_cache_subtitles_dir() +
+                        sep + movie_name + '.' + ext)
                 try:
                     old_subtitle_file.delete (None)
-                except:
+                except: # pylint: disable=bare-except
                     pass
 
                 # In the movie dir
-                old_subtitle_file = Gio.file_new_for_path (self._movie_dir() + sep + movie_name + '.' + ext)
+                old_subtitle_file = Gio.file_new_for_path (self._movie_dir() +
+                        sep + movie_name + '.' + ext)
                 try:
                     old_subtitle_file.delete (None)
-                except:
+                except: # pylint: disable=bare-except
                     pass
 
             flags = Gio.FileCreateFlags.REPLACE_DESTINATION
             try:
-                subtitle_file = Gio.file_new_for_path (self._movie_dir() + sep + movie_name + '.' + extension)
-                print ('trying to save to ' + subtitle_file.get_uri())
+                subtitle_file = Gio.file_new_for_path (self._movie_dir() +
+                        sep + movie_name + '.' + extension)
                 suburi = subtitle_file.get_uri ()
 
                 sub_file = subtitle_file.replace ('', False, flags, None)
                 sub_file.write (subtitles, None)
                 sub_file.close (None)
-            except:
-                subtitle_file = Gio.file_new_for_path (self._cache_subtitles_dir() + sep + movie_name + '.' + extension)
-                print ('saving to ' + subtitle_file.get_uri())
+            except: # pylint: disable=bare-except
+                subtitle_file = Gio.file_new_for_path (_cache_subtitles_dir() +
+                        sep + movie_name + '.' + extension)
                 suburi = subtitle_file.get_uri ()
 
                 sub_file = subtitle_file.replace ('', False, flags, None)
@@ -673,7 +685,7 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         self._close_dialog ()
 
         if suburi:
-            self._totem.set_current_subtitle (suburi)
+            self._totem.set_current_subtitle (suburi) # pylint: disable=no-member
 
         return False
 
@@ -722,9 +734,9 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
             self._apply_button.set_sensitive (False)
 
     def __on_treeview__row_activate (self,
-                                     _tree_path, # pylint: disable-msg=W0613
-                                     _column, # pylint: disable-msg=W0613
-                                     _data): # pylint: disable-msg=W0613
+                                     _tree_path, # pylint: disable=W0613
+                                     _column, # pylint: disable=W0613
+                                     _data): # pylint: disable=W0613
         self._download_and_apply ()
 
     def __on_totem__file_opened (self, _, new_mrl):
@@ -769,8 +781,8 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
     def __on_find_clicked (self, _):
         self._apply_button.set_sensitive (False)
         self._find_button.set_sensitive (False)
-        self._filename = self._totem.get_current_mrl ()
+        self._filename = self._totem.get_current_mrl () # pylint: disable=no-member
         (movie_hash, movie_size) = hash_file (self._filename)
+        movie_title = self._totem.get_title_at_playlist_pos(self._totem.get_playlist_pos()) # pylint: disable=E1101
 
-        self._get_results (movie_hash, movie_size)
-
+        self._get_results (movie_hash, movie_size, movie_title)
